/*****************************************************************************

FILE
  __start_p1.c
  $Date: 2010/08/06 15:51:52 $
  $Revision: 1.3 $

DESCRIPTION
  A minimal startup function for the second core of a MPC56xx family processor.

  This function initializes the stack to the values defined in the LCF file and
  uses the linker defined _SDA_BASE_ and _SDA2_BASE_ symbols to initialize
  r2 and r13 per (EABI). It then calls main_p1().

*/
#include <__ppc_eabi_init.h>
#include <__ppc_eabi_linker.h>	/* linker-generated symbol declarations */
#include "MPC5675K_HWInit.h"

/* These symbols are defined in the Linker configuration file, but are not in */
/* __ppc_eabi_linker.h, so we'll provide the declarations here.  Note that    */
/* _heap_*_p1 definitions are currently not being used.                       */ 
__declspec(section ".init") extern char	_stack_addr_p1[];
__declspec(section ".init") extern char _stack_end_p1[];
__declspec(section ".init") extern char	_heap_addr_p1[];
__declspec(section ".init") extern char _heap_end_p1[];


extern void main_p1();

#ifdef __cplusplus
extern "C" {
#endif

#pragma push
#pragma section code_type ".text_p1"
#pragma function_align 8 /* need to align for correct VLE bit in SSCM_DPMBOOT when booting second core */
#pragma force_active on

/* This macro allows to use C defined address with the inline assembler */
#define MAKE_HLI_COMPATIBLE(hli_name, c_expr) enum { hli_name=/*lint -e30*/((int)(c_expr)) };



/* Write one MMU Table Entry:               */
/* r3, r4, r5 and r6 must hold              */
/* the values of MAS0, MAS1, MAS2 and MAS3  */
__asm void WriteMMUTableEntry_p1( void )
{ 
nofralloc

    /* Write MMU Assist Register 0 (MAS0); SPR 624 */
    mtspr   624, r3
    /* Write MMU Assist Register 1 (MAS1); SPR 625 */
    mtspr   625, r4
    /* Write MMU Assist Register 2 (MAS2); SPR 626 */
    mtspr   626, r5
    /* Write MMU Assist Register 3 (MAS3); SPR 627 */
    mtspr   627, r6
    /* Write the table entry */
    tlbwe

    blr
}


__asm void __initMMU_p1(void)
{
MAKE_HLI_COMPATIBLE(RAM_MAS0_VALUE,MAS0_VALUE(0))
MAKE_HLI_COMPATIBLE(RAM_MAS1_VALUE,MAS1_VALUE(V_VALID, IPROT_PROTECTED, TID_GLOBAL, 0, TSIZE_4GB))
#if __option(vle) 
	MAKE_HLI_COMPATIBLE(RAM_MAS2_FLAGS,MAS2_FLAGS(SHARED_CACHE_STATE_NOT_USED, VLE_MODE, WRITE_BACK, CACHE_INHIBIT, MEM_COHERENCE_NREQ, NOT_GUARDED, BIG_ENDIAN ))
#else
	MAKE_HLI_COMPATIBLE(RAM_MAS2_FLAGS,MAS2_FLAGS(SHARED_CACHE_STATE_NOT_USED, BOOK_E_MODE, WRITE_BACK, CACHE_INHIBIT, MEM_COHERENCE_NREQ, NOT_GUARDED, BIG_ENDIAN ))
#endif
MAKE_HLI_COMPATIBLE(RAM_MAS3_FLAGS,MAS3_FLAGS(READ_WRITE_EXECUTE))

nofralloc

     mflr     r27

     /* load r3 with MAS0 value */
     lis r3,RAM_MAS0_VALUE@h
     ori r3,r3,RAM_MAS0_VALUE@l
     
     /* load r4 with MAS1 value */
     lis r4,RAM_MAS1_VALUE@h
     ori r4,r4,RAM_MAS1_VALUE@l
     
     /* clear r5 to get the base adress 0x0000_0000 */       
     xor r5,r5,r5
     
     /* copy the base ram address to r6 also */
     mr r6,r5
     
     ori r5,r5,RAM_MAS2_FLAGS@l
     ori r6,r6,RAM_MAS3_FLAGS@l
     
     /* write the MMU entry defined through r3, r4, r5 and r6 */ 
     bl WriteMMUTableEntry_p1 

     mtlr r27

     blr
}


#ifdef ROM_VERSION 
#define P1_SRAM_BASE	0x50000000
#define P1_SRAM_LOOP	2048
#else
// in Debug mode, adjust acoreding to LCF to prevent the program from deleting itslef
#define P1_SRAM_BASE	0x50007000
#define P1_SRAM_LOOP	1936
#endif
asm extern void __start_p1(void)
{
	nofralloc
	
	/* Initialize MMU */
	mflr     r26;
	bl __initMMU_p1;
	mtlr r26;


	/* MPC567xK L2SRAM initialization code 
    	memory map 0x50000000 - 0x5003FFFF - 256KB
    */
    lis r11,P1_SRAM_BASE@h /* Base address of the L2SRAM, 64-bit word aligned */
    ori r11,r11,P1_SRAM_BASE@l
    li r12,P1_SRAM_LOOP /* Loop counter to get all of low L2SRAM; 256k/4 bytes/32 GPRs = 2048 */
    mtctr r12

	init_l2sram_loop_1:
		stmw r0,0(r11)        /* Write all 32 GPRs to L2SRAM */
		addi r11,r11,128      /* Inc the ram ptr; 32 GPRs * 4 bytes = 128 */
		bdnz init_l2sram_loop_1 /* Loop for all L2SRAM */

	/* Initialize stack pointer				*/
	lis	r1, _stack_addr_p1@ha
	/* Emulate a calling function's frame, for correct LR save in main_p1 */
	addi	r1, r1, (_stack_addr_p1@l) - 16

	/* Note that we are assuming small data is shared between the cores */
	/* Initialize small data area pointers (EABI)                       */
	lis   r2, _SDA2_BASE_@ha
	addi  r2, r2, _SDA2_BASE_@l
  
	lis   r13, _SDA_BASE_@ha
	addi  r13, r13, _SDA_BASE_@l

	/* now call z0's main program (using short branch)... */ 
	bl main_p1
	
	/* If main_p1 must be addressed using FAR_ADDRESSING, use:
	lis r6,main_p1@ha
	ori r6,r6,main_p1@l
	mtlr r6
	blrl
	*/

  	/* if main returns, just hang here */
here:   b here

	blr
}

#pragma pop

#ifdef __cplusplus
}
#endif

